from my_app_name.module.auth.service.user.repository.user_repository_factory import (
    user_repository,
)
from my_app_name.module.auth.service.user.user_service import UserService

user_service = UserService(user_repository=user_repository)



@app.get("/api/v1/my_entities", response_model=list[MyEntityResponse])
async def get_all_my_entities() -> MyEntityResponse:
    return await my_module_client.get_all_my_entities()


@app.get("/api/v1/my_entities/{my_entity_id}", response_model=MyEntityResponse)
async def get_my_entity_by_id(my_entity_id: str) -> MyEntityResponse:
    return await my_module_client.get_my_entity_by_id(my_entity_id)


@app.post(
    "/api/v1/my_entities", response_model=MyEntityResponse | list[MyEntityResponse]
)
async def create_my_entity(data: MyEntityCreate | list[MyEntityCreate]):
    if isinstance(data, MyEntityCreate):
        data_dict = data.model_dump(exclude_unset=True)
        audited_data = MyEntityCreateWithAudit(**data_dict, created_by="system")
        return await my_module_client.create_my_entity(audited_data)
    audited_data = [
        MyEntityCreateWithAudit(
            **row.model_dump(exclude_unset=True), created_by="system"
        )
        for row in data
    ]
    return await my_module_client.create_my_entity(audited_data)


@app.put("/api/v1/my_entities/{my_entity_id}", response_model=MyEntityResponse)
async def update_my_entity(my_entity_id: str, data: MyEntityUpdate) -> MyEntityResponse:
    data_dict = data.model_dump(exclude_unset=True)
    audited_data = MyEntityUpdateWithAudit(**data_dict, updated_by="system")
    return await my_module_client.update_my_entity(my_entity_id, audited_data)


@app.delete("/api/v1/my_entities/{my_entity_id}", response_model=MyEntityResponse)
async def delete_my_entity(my_entity_id: str) -> MyEntityResponse:
    return await my_module_client.delete_my_entity(my_entity_id, deleted_by="system")



from my_app_name.config import APP_AUTH_BASE_URL
from my_app_name.module.auth.client.any_client import AnyClient
from my_app_name.module.auth.service.user.user_service_factory import user_service


class APIClient(user_service.as_api_client(base_url=APP_AUTH_BASE_URL), AnyClient):
    pass



from fastapi import FastAPI
from my_app_name.module.auth.client.factory import client as auth_client
from my_app_name.schema.user import (
    UserCreate,
    UserCreateWithAudit,
    UserResponse,
    UserUpdate,
    UserUpdateWithAudit,
)


def serve_auth_route(app: FastAPI):

    @app.get("/api/v1/users", response_model=list[UserResponse])
    async def get_all_users() -> UserResponse:
        return await auth_client.get_all_users()

    @app.get("/api/v1/users/{user_id}", response_model=UserResponse)
    async def get_user_by_id(user_id: str) -> UserResponse:
        return await auth_client.get_user_by_id(user_id)

    @app.post("/api/v1/users", response_model=UserResponse | list[UserResponse])
    async def create_user(data: UserCreate | list[UserCreate]):
        if isinstance(data, UserCreate):
            data_dict = data.model_dump(exclude_unset=True)
            audited_data = UserCreateWithAudit(**data_dict, created_by="system")
            return await auth_client.create_user(audited_data)
        audited_data = [
            UserCreateWithAudit(
                **row.model_dump(exclude_unset=True), created_by="system"
            )
            for row in data
        ]
        return await auth_client.create_user(audited_data)

    @app.put("/api/v1/users/{user_id}", response_model=UserResponse)
    async def update_user(user_id: str, data: UserUpdate) -> UserResponse:
        data_dict = data.model_dump(exclude_unset=True)
        audited_data = UserUpdateWithAudit(**data_dict, updated_by="system")
        return await auth_client.update_user(user_id, audited_data)

    @app.delete("/api/v1/users/{user_id}", response_model=UserResponse)
    async def delete_user(user_id: str) -> UserResponse:
        return await auth_client.delete_user(user_id, deleted_by="system")



import os
from typing import Any

from fastapi.responses import HTMLResponse
from my_app_name.common.view import render_page, render_str
from my_app_name.config import (
    APP_GATEWAY_CSS_PATH_LIST,
    APP_GATEWAY_FAVICON_PATH,
    APP_GATEWAY_JS_PATH_LIST,
    APP_GATEWAY_LOGO_PATH,
    APP_GATEWAY_SUBTITLE,
    APP_GATEWAY_TITLE,
    APP_GATEWAY_VIEW_DEFAULT_TEMPLATE_PATH,
    APP_GATEWAY_VIEW_PATH,
)

_DEFAULT_TEMPLATE_PATH = os.path.join(
    APP_GATEWAY_VIEW_PATH, APP_GATEWAY_VIEW_DEFAULT_TEMPLATE_PATH
)
_DEFAULT_ERROR_TEMPLATE_PATH = os.path.join(
    APP_GATEWAY_VIEW_PATH, "content", "error.html"
)

_DEFAULT_PARTIALS = {
    "title": APP_GATEWAY_TITLE,
    "subtitle": APP_GATEWAY_SUBTITLE,
    "logo_path": APP_GATEWAY_LOGO_PATH,
    "favicon_path": APP_GATEWAY_FAVICON_PATH,
    "css_path_list": APP_GATEWAY_CSS_PATH_LIST,
    "js_path_list": APP_GATEWAY_JS_PATH_LIST,
}


def render(
    view_path: str,
    template_path: str = _DEFAULT_TEMPLATE_PATH,
    status_code: int = 200,
    headers: dict[str, str] = None,
    media_type: str | None = None,
    partials: dict[str, Any] = {},
    **data: Any
) -> HTMLResponse:
    rendered_partials = {key: val for key, val in _DEFAULT_PARTIALS.items()}
    for key, val in partials.items():
        rendered_partials[key] = val
    return render_page(
        template_path=template_path,
        status_code=status_code,
        headers=headers,
        media_type=media_type,
        partials=rendered_partials,
        content=render_str(template_path=view_path, **data),
    )


def render_error(
    error_message: str,
    status_code: int = 500,
    view_path: str = _DEFAULT_ERROR_TEMPLATE_PATH,
    template_path: str = _DEFAULT_TEMPLATE_PATH,
    headers: dict[str, str] = None,
    media_type: str | None = None,
    partials: dict[str, Any] = {},
):
    return render(
        view_path=view_path,
        template_path=template_path,
        status_code=status_code,
        headers=headers,
        media_type=media_type,
        partials=partials,
        error_status_code=status_code,
        error_message=error_message,
    )



from my_app_name.config import APP_COMMUNICATION
from my_app_name.module.my_module.client.any_client import AnyClient
from my_app_name.module.my_module.client.api_client import APIClient
from my_app_name.module.my_module.client.direct_client import DirectClient

if APP_COMMUNICATION == "direct":
    client: AnyClient = DirectClient()
elif APP_COMMUNICATION == "api":
    client: AnyClient = APIClient()



from abc import ABC, abstractmethod

from my_app_name.schema.user import (
    User,
    UserCreateWithAudit,
    UserResponse,
    UserUpdateWithAudit,
)


class UserRepository(ABC):
    @abstractmethod
    async def create(self, user_data: UserCreateWithAudit) -> UserResponse:
        pass

    @abstractmethod
    async def get_by_id(self, user_id: str) -> UserResponse:
        pass

    @abstractmethod
    async def get_all(self) -> list[User]:
        pass

    @abstractmethod
    async def update(
        self, user_id: str, user_data: UserUpdateWithAudit
    ) -> UserResponse:
        pass

    @abstractmethod
    async def delete(self, user_id: str) -> UserResponse:
        pass

    @abstractmethod
    async def create_bulk(
        self, user_data_list: list[UserCreateWithAudit]
    ) -> list[UserResponse]:
        pass

    @abstractmethod
    async def get_by_credentials(self, username: str, password: str) -> UserResponse:
        pass



from my_app_name._zrb.format_task import format_my_app_name_code
from my_app_name._zrb.group import app_create_group

from zrb import Task

add_my_app_name_column = app_create_group.add_task(
    Task(
        name="add-my-app-name-column",
        description="📊 Create new column on an entity",
        successor=format_my_app_name_code,
        retries=0,
    ),
    alias="column",
)



import os

from fastapi import FastAPI, HTTPException, Request
from fastapi.exception_handlers import http_exception_handler
from fastapi.responses import HTMLResponse
from my_app_name.common.app import app
from my_app_name.common.schema import BasicResponse
from my_app_name.config import (
    APP_GATEWAY_VIEW_PATH,
    APP_MAIN_MODULE,
    APP_MODE,
    APP_MODULES,
)
from my_app_name.module.gateway.subroute.auth import serve_auth_route
from my_app_name.module.gateway.util.view import render, render_error


def serve_route(app: FastAPI):
    if APP_MODE != "monolith" and "gateway" not in APP_MODULES:
        return
    if APP_MODE == "monolith" or APP_MAIN_MODULE == "gateway":
        _serve_health_check(app)
        _serve_readiness_check(app)
        _serve_homepage(app)
        _handle_404(app)

    # Serve auth routes
    serve_auth_route(app)


def _serve_homepage(app: FastAPI):
    @app.get("/", include_in_schema=False)
    def home_page():
        return render(
            view_path=os.path.join(APP_GATEWAY_VIEW_PATH, "content", "homepage.html")
        )


def _serve_health_check(app: FastAPI):
    @app.api_route("/health", methods=["GET", "HEAD"], response_model=BasicResponse)
    async def health():
        """
        My App Name's health check
        """
        return BasicResponse(message="ok")


def _serve_readiness_check(app: FastAPI):
    @app.api_route("/readiness", methods=["GET", "HEAD"], response_model=BasicResponse)
    async def readiness():
        """
        My App Name's readiness check
        """
        return BasicResponse(message="ok")


def _handle_404(app: FastAPI):
    @app.exception_handler(404)
    async def default_404(request: Request, exc: HTTPException) -> HTMLResponse:
        if request.url.path.startswith("/api"):
            # Re-raise the exception to let FastAPI handle it
            return await http_exception_handler(request, exc)
        return render_error(error_message="Not found", status_code=404)


serve_route(app)



import os
from contextlib import asynccontextmanager

from fastapi import FastAPI, HTTPException
from fastapi.openapi.docs import get_swagger_ui_html
from fastapi.responses import FileResponse
from fastapi.staticfiles import StaticFiles
from my_app_name.common.db_engine import engine
from my_app_name.config import (
    APP_GATEWAY_FAVICON_PATH,
    APP_GATEWAY_TITLE,
    APP_GATEWAY_VIEW_PATH,
    APP_MODE,
    APP_MODULES,
    APP_VERSION,
)
from sqlmodel import SQLModel


@asynccontextmanager
async def lifespan(app: FastAPI):
    SQLModel.metadata.create_all(engine)
    yield


app_title = (
    APP_GATEWAY_TITLE
    if APP_MODE == "monolith"
    else f"{APP_GATEWAY_TITLE} - {', '.join(APP_MODULES)}"
)
app = FastAPI(
    title=app_title,
    version=APP_VERSION,
    lifespan=lifespan,
    docs_url=None,
)

_STATIC_DIR = os.path.join(APP_GATEWAY_VIEW_PATH, "static")
app.mount("/static", StaticFiles(directory=_STATIC_DIR), name="static")


# Serve static files
@app.get("/static/{file_path:path}", include_in_schema=False)
async def static_files(file_path: str):
    full_path = os.path.join(_STATIC_DIR, file_path)
    if os.path.isfile(full_path):
        return FileResponse(full_path)
    raise HTTPException(status_code=404, detail="File not found")


@app.get("/docs", include_in_schema=False)
async def swagger_ui_html():
    return get_swagger_ui_html(
        openapi_url="/openapi.json",
        title=app_title,
        swagger_favicon_url=APP_GATEWAY_FAVICON_PATH,
    )



@abstractmethod
async def get_my_entity_by_id(self, my_entity_id: str) -> MyEntityResponse:
    pass


@abstractmethod
async def get_all_my_entities(self) -> list[MyEntityResponse]:
    pass


@abstractmethod
async def create_my_entity(
    self, data: MyEntityCreateWithAudit | list[MyEntityCreateWithAudit]
) -> MyEntityResponse | list[MyEntityResponse]:
    pass


@abstractmethod
async def update_my_entity(
    self, my_entity_id: str, data: MyEntityUpdateWithAudit
) -> MyEntityResponse:
    pass


@abstractmethod
async def delete_my_entity(self, my_entity_id: str) -> MyEntityResponse:
    pass



import os

from my_app_name._zrb.config import APP_DIR
from my_app_name._zrb.entity.add_entity_util import (
    is_in_app_schema_dir,
    is_in_module_entity_dir,
    is_module_any_client_file,
    is_module_api_client_file,
    is_module_direct_client_file,
    is_module_gateway_subroute_file,
    is_module_migration_metadata_file,
    is_module_route_file,
    update_any_client,
    update_api_client,
    update_direct_client,
    update_gateway_subroute,
    update_migration_metadata,
    update_route,
)
from my_app_name._zrb.format_task import format_my_app_name_code
from my_app_name._zrb.group import app_create_group
from my_app_name._zrb.input import (
    existing_module_input,
    new_entity_column_input,
    new_entity_input,
    plural_entity_input,
)
from my_app_name._zrb.util import get_existing_module_names, get_existing_schema_names

from zrb import AnyContext, ContentTransformer, Scaffolder, Task, make_task
from zrb.util.string.conversion import to_snake_case


@make_task(
    name="validate-create-my-app-name-entity",
    input=[
        existing_module_input,
        new_entity_input,
        plural_entity_input,
        new_entity_column_input,
    ],
    retries=0,
)
async def validate_create_my_app_name_entity(ctx: AnyContext):
    module_name = to_snake_case(ctx.input.module)
    if module_name not in get_existing_module_names():
        raise ValueError(f"Module not exist: {module_name}")
    schema_name = to_snake_case(ctx.input.entity)
    if schema_name in get_existing_schema_names():
        raise ValueError(f"Schema already exists: {schema_name}")


scaffold_my_app_name_entity = Scaffolder(
    name="scaffold-my-app-name-entity",
    input=[
        existing_module_input,
        new_entity_input,
        plural_entity_input,
        new_entity_column_input,
    ],
    source_path=os.path.join(os.path.dirname(__file__), "template", "app_template"),
    render_source_path=False,
    destination_path=APP_DIR,
    transform_path={
        "my_module": "{to_snake_case(ctx.input.module)}",
        "my_entity": "{to_snake_case(ctx.input.entity)}",
    },
    transform_content=[
        # Schema tranformation (my_app_name/schema/snake_entity_name)
        ContentTransformer(
            name="transform-schema-file",
            match=is_in_app_schema_dir,
            transform={
                "MyEntity": "{to_pascal_case(ctx.input.entity)}",
                "my_column": "{to_snake_case(ctx.input.column)}",
            },
        ),
        # Common module's entity transformation
        # (my_app_name/module/snake_module_name/service/snake_entity_name)
        ContentTransformer(
            name="transform-module-entity-dir",
            match=is_in_module_entity_dir,
            transform={
                "my_module": "{to_snake_case(ctx.input.module)}",
                "MyEntity": "{to_pascal_case(ctx.input.entity)}",
                "my_entity": "{to_snake_case(ctx.input.entity)}",
                "my_entities": "{to_snake_case(ctx.input.plural)}",
                "my-entities": "{to_kebab_case(ctx.input.plural)}",
            },
        ),
        # Add entity to migration metadata
        # (my_app_name/module/snake_module_name/migration_metadata.py)
        ContentTransformer(
            name="transform-module-migration-metadata",
            match=is_module_migration_metadata_file,
            transform=update_migration_metadata,
        ),
        # Update API Client (my_app_name/module/snake_module_name/client/api_client.py)
        ContentTransformer(
            name="transform-module-api-client",
            match=is_module_api_client_file,
            transform=update_api_client,
        ),
        # Update Direct Client (my_app_name/module/snake_module_name/client/direct_client.py)
        ContentTransformer(
            name="transform-module-direct-client",
            match=is_module_direct_client_file,
            transform=update_direct_client,
        ),
        # Update Any Client (my_app_name/module/snake_module_name/client/any_client.py)
        ContentTransformer(
            name="transform-module-any-client",
            match=is_module_any_client_file,
            transform=update_any_client,
        ),
        # Update module route (my_app_name/module/route.py)
        ContentTransformer(
            name="transform-module-route",
            match=is_module_route_file,
            transform=update_route,
        ),
        # Update module gateway subroute
        # (my_app_name/module/gateway/subroute/snake_module_name.py)
        ContentTransformer(
            name="transform-module-gateway-subroute",
            match=is_module_gateway_subroute_file,
            transform=update_gateway_subroute,
        ),
    ],
    retries=0,
    upstream=validate_create_my_app_name_entity,
)

add_my_app_name_entity = app_create_group.add_task(
    Task(
        name="add-my-app-name-entity",
        description="🏗️ Create new entity on a module",
        upstream=scaffold_my_app_name_entity,
        successor=format_my_app_name_code,
        retries=0,
    ),
    alias="entity",
)



from my_app_name.common.base_service import BaseService
from my_app_name.module.auth.service.user.repository.user_repository import (
    UserRepository,
)
from my_app_name.schema.user import (
    UserCreateWithAudit,
    UserResponse,
    UserUpdateWithAudit,
)


class UserService(BaseService):
    def __init__(self, user_repository: UserRepository):
        super().__init__()
        self.user_repository = user_repository

    @BaseService.route(
        "/api/v1/users/{user_id}", methods=["get"], response_model=UserResponse
    )
    async def get_user_by_id(self, user_id: str) -> UserResponse:
        return await self.user_repository.get_by_id(user_id)

    @BaseService.route(
        "/api/v1/users", methods=["get"], response_model=list[UserResponse]
    )
    async def get_all_users(self) -> list[UserResponse]:
        return await self.user_repository.get_all()

    @BaseService.route(
        "/api/v1/users",
        methods=["post"],
        response_model=UserResponse | list[UserResponse],
    )
    async def create_user(
        self, data: UserCreateWithAudit | list[UserCreateWithAudit]
    ) -> UserResponse | list[UserResponse]:
        if isinstance(data, UserCreateWithAudit):
            return await self.user_repository.create(data)
        return await self.user_repository.create_bulk(data)

    @BaseService.route(
        "/api/v1/users/{user_id}", methods=["put"], response_model=UserResponse
    )
    async def update_user(
        self, user_id: str, data: UserUpdateWithAudit
    ) -> UserResponse:
        return await self.user_repository.update(user_id, data)

    @BaseService.route(
        "/api/v1/users/{user_id}", methods=["delete"], response_model=UserResponse
    )
    async def delete_user(self, user_id: str, deleted_by: str) -> UserResponse:
        return await self.user_repository.delete(user_id)



from abc import ABC, abstractmethod

from my_app_name.schema.my_entity import (
    MyEntity,
    MyEntityCreateWithAudit,
    MyEntityResponse,
    MyEntityUpdateWithAudit,
)


class MyEntityRepository(ABC):
    @abstractmethod
    async def create(self, my_entity_data: MyEntityCreateWithAudit) -> MyEntityResponse:
        pass

    @abstractmethod
    async def get_by_id(self, my_entity_id: str) -> MyEntity:
        pass

    @abstractmethod
    async def get_all(self) -> list[MyEntity]:
        pass

    @abstractmethod
    async def update(
        self, my_entity_id: str, my_entity_data: MyEntityUpdateWithAudit
    ) -> MyEntity:
        pass

    @abstractmethod
    async def delete(self, my_entity_id: str) -> MyEntity:
        pass

    @abstractmethod
    async def create_bulk(
        self, my_entity_data_list: list[MyEntityCreateWithAudit]
    ) -> list[MyEntityResponse]:
        pass



from my_app_name.module.auth.client.any_client import AnyClient
from my_app_name.module.auth.service.user.user_service_factory import user_service


class DirectClient(user_service.as_direct_client(), AnyClient):
    pass
